﻿using System;
//using System.Collections.Generic;
//using System.Linq;
using System.Text; 
using System.Net; // IPHostEntry, IPAddress, etc.
using System.Net.Sockets; // Socket


namespace WCFTestHarness
{
  class Program
  {
    static void Main(string[] args)
    {
      try
      {
        Console.WriteLine("\nBegin WCF MathService testing via sockets run");

        string[] testCases = new string[] {
          "001,3.4,5.7,9.1",
          "002,0.0,0.1,1.0",
          "003,6.7,6.7,13.4"
        };
        
        foreach (string testCase in testCases)
        {
          Console.WriteLine("\n=======================================\n");
          string[] tokens = testCase.Split(',');
          string caseID = tokens[0];
          string input1 = tokens[1];
          string input2 = tokens[2];
          string expected = tokens[3];

          string soapMessage = "<s:Envelope xmlns:s='http://www.w3.org/2003/05/soap-envelope'";
          soapMessage += " xmlns:a='http://www.w3.org/2005/08/addressing'><s:Header>";
          soapMessage += "<a:Action s:mustUnderstand='1'>http://tempuri.org/IMathService/Sum</a:Action>";
          soapMessage += "<a:MessageID>urn:uuid:510b1790-0b89-4c85-8015-d1043ffeea14</a:MessageID>";
          soapMessage += "<a:ReplyTo><a:Address>http://www.w3.org/2005/08/addressing/anonymous</a:Address>";
          soapMessage += "</a:ReplyTo><a:To s:mustUnderstand='1'>";
          soapMessage += "http://localhost:8000/MyWCFMathService/Service/MathService</a:To></s:Header>";
          soapMessage += "<s:Body><Sum xmlns='http://tempuri.org/'>";

          soapMessage += "<x>" + input1 + "</x>" + "<y>" + input2 + "</y>";

          soapMessage += "</Sum></s:Body></s:Envelope>";

          Console.WriteLine("Test Case   : " + caseID);

          Console.WriteLine("Input       : <s:Envelope..." + "<x>" + input1 + "</x>" + "<y>" + input2 + "</y>...</s:Envelope>");
          Console.WriteLine("Expected    : " + expected);

          string host = "localhost";
          IPHostEntry iphe = Dns.Resolve(host);
          IPAddress[] addList = iphe.AddressList;
          EndPoint ep = new IPEndPoint(addList[0], 8000);
          

          Socket socket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);

          socket.Connect(ep);
          if (socket.Connected) Console.WriteLine("Connected to socket at " + ep.ToString());
          else Console.WriteLine("Error: Unable to connect to " + ep.ToString());

          string header = "POST /MyWCFMathService/Service/MathService HTTP/1.1\r\n";
          header += "Content-Type: application/soap+xml; charset=utf-8\r\n";
          header += "SOAPAction: 'http://tempuri.org/IMathService/Sum'\r\n";
          header += "Host: localhost:8000\r\n";
          header += "Content-Length: " + soapMessage.Length + "\r\n";
          header += "Expect: 100-continue\r\n";
          //header += "Connection: Keep-Alive\r\n\r\n";
          header += "Connection: Close\r\n\r\n";

          string sendAsString = header + soapMessage;
          byte[] sendAsBytes = Encoding.UTF8.GetBytes(sendAsString);
          Console.WriteLine("Sending input to WCF service");
          int numBytesSent = socket.Send(sendAsBytes, sendAsBytes.Length, SocketFlags.None);
          
          byte[] receivedBufferAsBytes = new byte[512];
          string receiveAsString = "";
          string entireReceive = "";
          int numBytesReceived = 0;

          //Console.WriteLine("receiving");
          while ((numBytesReceived = socket.Receive(receivedBufferAsBytes, 512, SocketFlags.None)) > 0)
          {
            //Console.WriteLine("top of loop . . .");
            receiveAsString = Encoding.UTF8.GetString(receivedBufferAsBytes, 0, numBytesReceived);
            //Console.WriteLine(receiveAsString);
            entireReceive += receiveAsString;
          }

          //Console.WriteLine("\n\nReceived " + entireReceive);
          Console.WriteLine("Response received");

          if (entireReceive.IndexOf(expected) >= 0)
            Console.WriteLine("Test result : Pass");
          else
            Console.WriteLine("Test result : **FAIL**");

          //Console.WriteLine("Closing socket");
          //socket.Close();

        } // main loop
        Console.WriteLine("\n=======================================");

        Console.WriteLine("\nEnd test run");
        Console.ReadLine();

      }
      catch (Exception ex)
      {
        Console.WriteLine("Fatal: " + ex.Message);
        Console.ReadLine();
      }
    } // Main()
  } // Program
} // ns
